set.seed(1234567890)

library(geosphere)

stations <- read.csv("stations.csv", fileEncoding = "latin1")
temps <- read.csv("temps50k.csv")

st <- merge(stations, temps, by = "station_number")

h_distance <- 100000
h_date <- 15
h_time <- 4

a <- 58.2357
b <- 15.3437
date <- "1974-12-18"
times <- c(
  "04:00:00",
  "06:00:00",
  "08:00:00",
  "10:00:00",
  "12:00:00",
  "14:00:00",
  "16:00:00",
  "18:00:00",
  "20:00:00",
  "22:00:00",
  "24:00:00"
)


st$time <- strptime(st$time, format = "%H:%M:%S")

gaussian_kernel <- function(x, h) {
  exp(-(x ^ 2) / (2 * h ^ 2))
}

x = seq(0, 300000, length.out = 1000)
plot(
  gaussian_kernel(x, h_distance),
  type = 'l',
  xlab = "Physical distance",
  ylab = "Kernel value",
  main = "Gaussian distance kernel"
)
grid()

x = seq(0, 60)
plot(
  gaussian_kernel(x, h_date),
  type = 'l',
  xlab = "Distance in days",
  ylab = "Kernel value",
  main = "Gaussian date kernel"
)
grid()

x = seq(0, 18)
plot(
  gaussian_kernel(x, h_time),
  type = 'l',
  xlab = "Distance in hours",
  ylab = "Kernel value",
  main = "Gaussian time kernel"
)
grid()

temp_add <- c()
temp_mult <- c()

for (time in times) {
  time <- strptime(time, format = "%H:%M:%S")
  
  
  # Filter out posterior dates and time
  st_temp <- st[st$date < date |
                  (st$date == date &
                     st$time <= time), ]
  
  
  distance_kernels <- mapply(function(lat, lon) {
    dist <- distHaversine(c(a, b), c(lat, lon))
    gaussian_kernel(dist, h_distance)
  }, st_temp$latitude, st_temp$longitude)
  
  
  date_kernels <- mapply(function(x_i) {
    dist <- as.numeric(as.Date(date) - as.Date(x_i))
    gaussian_kernel(dist, h_date)
  }, st_temp$date)
  
  
  time_kernels <- mapply(function(x_i) {
    dist <- as.numeric(difftime(time, x_i, units = "hours"))
    gaussian_kernel(dist, h_time)
  }, st_temp$time)
  
  
  kernels_add <- distance_kernels + date_kernels + time_kernels
  kernels_mult <- distance_kernels * date_kernels * time_kernels
  
  temp_add <<- c(temp_add,
                 sum(kernels_add * st_temp$air_temperature) / sum(kernels_add))
  temp_mult <<- c(temp_mult,
                  sum(kernels_mult * st_temp$air_temperature) / sum(kernels_mult))
  
}

plot(
  temp_add,
  type = "o",
  xaxt = "n",
  xlab = "Time",
  ylab = "Temperature",
  main = "Added kernels"
)
axis(
  1,                              
  at = 1:length(times),         
  labels = times                  
)
grid()

plot(
  temp_mult,
  type = "o",
  xaxt = "n",
  xlab = "Time",
  ylab = "Temperature",
  main = "Multiplied kernels"
)
axis(
  1,                              
  at = 1:length(times),         
  labels = times                  
)
grid()